@@ -32,7 +32,7 @@ from .line import Line
 from .layout import Layout
 from .layout.prompt import DefaultPrompt
 from .renderer import Renderer
-from .utils import raw_mode, call_on_sigwinch
+from .utils import raw_mode, cooked_mode, call_on_sigwinch
 from .history import History
 
 from pygments.styles.default import DefaultStyle
@@ -90,7 +90,8 @@ class CommandLineInterface(object):
                  renderer_factory=Renderer,
                  default_input_mode=InputMode.INSERT,
                  style=DefaultStyle,
-                 key_binding_factories=None):
+                 key_binding_factories=None,
+                 async_autocomplete_on_text_insert=False):
 
         self.stdin = stdin or sys.stdin
         self.stdout = stdout or sys.stdout
@@ -103,6 +104,7 @@ class CommandLineInterface(object):
         self.lines = {
             'default': line,
             'search': Line(history=History()),
+            'system': Line(history=History()),
         }
 
         #: The `Layout` instance.
@@ -122,7 +124,7 @@ class CommandLineInterface(object):
         self.inputstream = InputStream(self.input_processor)
 
         # Pipe for inter thread communication.
-        self._redraw_pipe = None
+        self._schedule_pipe = None
 
         #: Currently reading input.
         self.is_reading_input = False
@@ -132,6 +134,13 @@ class CommandLineInterface(object):
         # it could be that we are in the middle of a utf-8 byte sequence.
         self._stdin_decoder = self.stdin_decoder_cls()
 
+        self._calls_from_executor = []
+
+        # Handle events.
+        if async_autocomplete_on_text_insert:
+            self.lines['default'].onTextInsert += self._async_autocomplete
+        self.complete_thread_running = False
+
         self._reset()
 
     @property
@@ -158,8 +167,7 @@ class CommandLineInterface(object):
         """
         Thread safe way of sending a repaint trigger to the input event loop.
         """
-        if self._redraw_pipe:
-            os.write(self._redraw_pipe[1], b'x')
+        self.call_from_executor(self._redraw)
 
     def _redraw(self):
         """
@@ -173,6 +181,7 @@ class CommandLineInterface(object):
         Run a long running function in a background thread.
         (This is recommended for code that could block the `read_input` event
         loop.)
+        Similar to Twisted's ``deferToThread``.
         """
         class _Thread(threading.Thread):
             def run(t):
@@ -180,6 +189,16 @@ class CommandLineInterface(object):
 
         _Thread().start()
 
+    def call_from_executor(self, callback):
+        """
+        Call this function in the main event loop.
+        Similar to Twisted's ``callFromThread``.
+        """
+        self._calls_from_executor.append(callback)
+
+        if self._schedule_pipe:
+            os.write(self._schedule_pipe[1], b'x')
+
     def on_input_timeout(self):
         """
         Called when there is no input for x seconds.
@@ -189,10 +208,10 @@ class CommandLineInterface(object):
         # function below the cursor position in the case of a REPL.
         pass
 
-    def on_read_input_start(self):
+    def on_read_input_start(self):  # XXX: replace with EventHook
         pass
 
-    def on_read_input_end(self):
+    def on_read_input_end(self):  # XXX: replace with EventHook
         pass
 
     def _get_char_loop(self):
@@ -204,19 +223,23 @@ class CommandLineInterface(object):
         timeout = self.input_timeout
 
         while True:
-            r, w, x = _select([self.stdin, self._redraw_pipe[0]], [], [], timeout)
+            r, w, x = _select([self.stdin, self._schedule_pipe[0]], [], [], timeout)
 
             if self.stdin in r:
                 return self._read_from_stdin()
 
-            # If we receive something on our redraw pipe, render again.
-            elif self._redraw_pipe[0] in r:
-                # Flush all the pipe content and repaint.
-                os.read(self._redraw_pipe[0], 1024)
-                self._redraw()
-                timeout = None
+            # If we receive something on our "call_from_executor" pipe, process
+            # these callbacks in a thread safe way.
+            elif self._schedule_pipe[0] in r:
+                # Flush all the pipe content.
+                os.read(self._schedule_pipe[0], 1024)
+
+                # Process calls from executor.
+                calls_from_executor, self._calls_from_executor = self._calls_from_executor, []
+                for c in calls_from_executor:
+                    c()
             else:
-                #
+                # Handle input timeout.
                 self.on_input_timeout()
                 timeout = None
 
@@ -260,8 +283,8 @@ class CommandLineInterface(object):
 
         try:
             # Create a pipe for inter thread communication.
-            self._redraw_pipe = os.pipe()
-            fcntl.fcntl(self._redraw_pipe[0], fcntl.F_SETFL, os.O_NONBLOCK)
+            self._schedule_pipe = os.pipe()
+            fcntl.fcntl(self._schedule_pipe[0], fcntl.F_SETFL, os.O_NONBLOCK)
 
             def reset_line():
                 """
@@ -340,13 +363,13 @@ class CommandLineInterface(object):
                         self._redraw()
 
         finally:
-            # Close redraw pipes.
-            redraw_pipe = self._redraw_pipe
-            self._redraw_pipe = None
+            # Close pipes.
+            schedule_pipe = self._schedule_pipe
+            self._schedule_pipe = None
 
-            if redraw_pipe:
-                os.close(redraw_pipe[0])
-                os.close(redraw_pipe[1])
+            if schedule_pipe:
+                os.close(schedule_pipe[0])
+                os.close(schedule_pipe[1])
 
             # Trigger read_end.
             self.on_read_input_end()
@@ -362,6 +385,21 @@ class CommandLineInterface(object):
     def set_return_value(self, code):
         self._return_code = code
 
+    def run_system_command(self, command):
+        """
+        (Not thread safe -- to be called from inside the key bindings.)
+        Run system command.
+        """
+        self.renderer.erase()
+
+        # Run system command.
+        with cooked_mode(self.stdin.fileno()):
+            os.system(command.encode('utf-8'))
+            raw_input('\nPress ENTER to continue...')
+
+        self.renderer.reset()
+        self.renderer.request_absolute_cursor_position()
+
     @property
     def is_exiting(self):
         return self._exit_flag
@@ -374,6 +412,53 @@ class CommandLineInterface(object):
     def is_returning(self):
         return self._return_code
 
+    def _async_autocomplete(self):
+        """
+        Asynchronous autocompletion while typing.
+        (Autocomplete in other thread.)
+        """
+        document = self.line.document
+
+        # Don't start two threads at the same time.
+        if self.complete_thread_running:
+            return
+
+        # Don't complete when we already have completions.
+        if self.line.complete_state:
+            return
+
+        # Don't automatically complete on empty inputs.
+        char = document.char_before_cursor
+        if not self.line.text or char.isspace():
+            return
+
+        # Otherwise, get completions in other thread.
+        self.complete_thread_running = True
+
+        def run():
+            completions = list(self.line.completer.get_completions(document))
+            self.complete_thread_running = False
+
+            def callback():
+                """
+                Set the new complete_state in a safe way. Don't replace an
+                existing complete_state if we had one. (The user could have
+                pressed 'Tab' in the meantime. Also don't set it if the text
+                was changed in the meantime.
+                """
+                # Set completions if the text was not yet changed.
+                if self.line.text == document.text and \
+                        self.line.cursor_position == document.cursor_position and \
+                        not self.line.complete_state:
+                    self.line._start_complete(go_to_first=False, completions=completions)
+                    self._redraw()
+                else:
+                    # Otherwise, restart thread.
+                    self._async_autocomplete()
+            self.call_from_executor(callback)
+
+        self.run_in_executor(run)
+
 
 def _select(*args, **kwargs):
     """
