@@ -23,7 +23,7 @@ from prompt_toolkit.keys import Keys
 from prompt_toolkit.layout import Layout
 from prompt_toolkit.layout.menus import CompletionMenu
 from prompt_toolkit.layout.processors import BracketsMismatchProcessor
-from prompt_toolkit.layout.toolbars import CompletionToolbar, ArgToolbar, SearchToolbar, ValidationToolbar
+from prompt_toolkit.layout.toolbars import CompletionToolbar, ArgToolbar, SearchToolbar, ValidationToolbar, SystemToolbar
 from prompt_toolkit.layout.toolbars import Toolbar
 from prompt_toolkit.layout.utils import TokenList
 from prompt_toolkit.line import Line
@@ -89,11 +89,13 @@ class PythonStyle(Style):
 
         # Highlighting for the reverse-search prompt.
         Token.Prompt:  'bold #008800',
-        Token.Layout.Toolbar.Search:             'noinherit',
-        Token.Layout.Toolbar.Search.Prefix:      '#6666aa noinherit',
+        Token.Layout.Toolbar.Search:             '#44ffff noinherit',
         Token.Layout.Toolbar.Search.Text:        'noinherit',
         Token.Layout.Toolbar.Search.Text.NoMatch: 'bg:#aa4444 #ffffff',
 
+        Token.Layout.Toolbar.Arg:             '#44ffff noinherit',
+        Token.Layout.Toolbar.Arg.TExt:        'noinherit',
+
         Token.Prompt.SecondLinePrefix: 'bold #888888',
         Token.Prompt.LineNumber:       '#aa6666',
         Token.Prompt.Arg:              'noinherit',
@@ -265,15 +267,6 @@ class PythonLine(Line):
         """ Ignore setter. """
         pass
 
-    def complete_after_insert_text(self):
-        """
-        Start autocompletion when a we have a valid identifier before the
-        cursor. (In this case it's not required to press [Tab] in order to view
-        the completion menu.)
-        """
-        word_before_cursor = self.document.get_word_before_cursor()
-        return word_before_cursor is not None and _identifier_re.match(word_before_cursor)
-
 
 class SignatureToolbar(Toolbar):
     def is_visible(self, cli):
@@ -446,14 +439,6 @@ class PythonCompleter(Completer):
         self._globals = globals
         self._locals = locals
 
-    def complete_after_insert_text(self, document):
-        """
-        Open completion menu when we type a character.
-        (Except if we typed whitespace.)
-        """
-        char = document.char_before_cursor
-        return not char.isspace() and not char.isdigit() and not char in '()+-*/,:;\'"={}[]%^&#@!'
-
     def get_completions(self, document):
         """ Ask jedi to complete. """
         script = get_jedi_script_from_document(document, self._locals, self._globals)
@@ -482,7 +467,7 @@ class PythonCommandLineInterface(CommandLineInterface):
         self.always_multiline = always_multiline
         self.autocompletion_style = autocompletion_style
 
-        completer = _completer or PythonCompleter(self.globals, self.locals)
+        self.completer = _completer or PythonCompleter(self.globals, self.locals)
         validator = _validator or PythonValidator()
 
         layout = Layout(
@@ -495,6 +480,7 @@ class PythonCommandLineInterface(CommandLineInterface):
                 ArgToolbar(),
                 SignatureToolbar(),
                 SearchToolbar(),
+                SystemToolbar(),
                 ValidationToolbar(),
             ] +
             ([CompletionToolbar()] if autocompletion_style == AutoCompletionStyle.HORIZONTAL_MENU else []) +
@@ -513,6 +499,12 @@ class PythonCommandLineInterface(CommandLineInterface):
         else:
             key_binding_factories = [emacs_bindings, python_bindings]
 
+        line=PythonLine(always_multiline=always_multiline,
+                        tempfile_suffix='.py',
+                        history=history,
+                        completer=self.completer,
+                        validator=validator)
+
         #: Incremeting integer counting the current statement.
         self.current_statement_index = 1
 
@@ -522,11 +514,8 @@ class PythonCommandLineInterface(CommandLineInterface):
             layout=layout,
             style=style,
             key_binding_factories=key_binding_factories,
-            line=PythonLine(always_multiline=always_multiline,
-                            tempfile_suffix='.py',
-                            history=history,
-                            completer=completer,
-                            validator=validator))
+            line=line,
+            async_autocomplete_on_text_insert=True)
 
     def on_input_timeout(self):
         """
