@@ -31,8 +31,10 @@ def get_size(fileno):
 
 class raw_mode(object):
     """
-    with raw_mode(stdin):
-        ''' the pseudo-terminal stdin is now used in raw mode '''
+    ::
+
+        with raw_mode(stdin):
+            ''' the pseudo-terminal stdin is now used in raw mode '''
     """
     def __init__(self, fileno):
         self.fileno = fileno
@@ -41,14 +43,27 @@ class raw_mode(object):
     def __enter__(self):
         # NOTE: On os X systems, using pty.setraw() fails. Therefor we are using this:
         newattr = termios.tcgetattr(self.fileno)
-        newattr[tty.LFLAG] = newattr[tty.LFLAG] & ~(
-            termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
+        newattr[tty.LFLAG] = self._patch(newattr[tty.LFLAG])
         termios.tcsetattr(self.fileno, termios.TCSANOW, newattr)
 
+    def _patch(self, attrs):
+        return attrs & ~(termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
+
     def __exit__(self, *a, **kw):
         termios.tcsetattr(self.fileno, termios.TCSANOW, self.attrs_before)
 
 
+class cooked_mode(raw_mode):
+    """
+    (The opposide of ``raw_mode``::
+
+        with cooked_mode(stdin):
+            ''' the pseudo-terminal stdin is now used in cooked mode. '''
+    """
+    def _patch(self, attrs):
+        return attrs | (termios.ECHO | termios.ICANON | termios.IEXTEN | termios.ISIG)
+
+
 class call_on_sigwinch(object):
     """
     Context manager which Installs a SIGWINCH callback.
@@ -62,3 +77,31 @@ class call_on_sigwinch(object):
 
     def __exit__(self, *a, **kw):
         signal.signal(signal.SIGWINCH, self.previous_callback)
+
+
+
+class EventHook(object):
+    """
+    Event hook::
+
+        e = EventHook()
+        e += handler_function  # Add event handler.
+        e.fire()  # Fire event.
+
+    Thanks to Michael Foord:
+    http://www.voidspace.org.uk/python/weblog/arch_d7_2007_02_03.shtml#e616
+    """
+    def __init__(self):
+        self.__handlers = []
+
+    def __iadd__(self, handler):
+        self.__handlers.append(handler)
+        return self
+
+    def __isub__(self, handler):
+        self.__handlers.remove(handler)
+        return self
+
+    def fire(self, *args, **keywargs):
+        for handler in self.__handlers:
+            handler(*args, **keywargs)
